Package: JMbdirect
Type: Package
Title: Joint Model for Longitudinal and Multiple Time to Events Data
Version: 0.1.1
Imports: JMbayes2,joineRML,FastJM,rstanarm,dplyr,jmBIG
Authors@R: person(("Atanu"), "Bhattacharjee",
                  email="atanustat@gmail.com",
                  role=c("aut", "cre"))
Maintainer: Atanu Bhattacharjee <atanustat@gmail.com>
Description: Provides model fitting, prediction, and plotting for joint models of longitudinal and multiple time-to-event data, including methods from Rizopoulos (2012) <doi:10.1201/b12208>. Useful for handling complex survival and longitudinal data in clinical research.
License: GPL-3
Encoding: UTF-8
LazyData: true
Author: Atanu Bhattacharjee [aut, cre]
RoxygenNote: 7.3.2
Depends: R (>= 2.10)
NeedsCompilation: no
Packaged: 2026-05-22 07:29:55 UTC; root
Repository: CRAN
Date/Publication: 2026-05-22 08:02:03 UTC
Built: R 4.4.3; ; 2026-05-25 00:40:55 UTC; unix
