# Generated by roxygen2: do not edit by hand

S3method(plot,jmbB)
S3method(plot,jmcsB)
S3method(plot,jmrmlB)
S3method(plot,jmstB)
S3method(predict,jmbB)
S3method(predict,jmcsB)
S3method(predict,jmrmlB)
S3method(predict,jmstB)
S3method(print,jmbB)
S3method(print,jmcsB)
S3method(print,jmrmlB)
S3method(print,jmstB)
export(bootciJMCS)
export(jmbB)
export(jmcsB)
export(jmrmlB)
export(jmstB)
import(JMbayes2)
import(dplyr)
import(grDevices)
import(graphics)
import(jmBIG)
import(joineRML)
import(rstanarm)
importFrom(FastJM,jmcs)
importFrom(FastJM,survfitjmcs)
importFrom(JMbayes2,jm)
importFrom(rstanarm,VarCorr)
importFrom(rstanarm,stan_jm)
importFrom(stats,complete.cases)
importFrom(stats,median)
importFrom(stats,predict)
importFrom(stats,quantile)
